# Usage:
#   art/test/run-test --host --gdb [--64] [--interpreter] 004-JniTest
#   'b Java_Main_shortMethod'
#   'r'
#   'command script import host_art_bt.py'
#   'host_art_bt'

import sys
import re

import lldb


def host_art_bt(debugger, command, result, internal_dict):
    prettified_frames = []
    lldb_frame_index = 0
    art_frame_index = 0
    target = debugger.GetSelectedTarget()
    process = target.GetProcess()
    thread = process.GetSelectedThread()
    while lldb_frame_index < thread.GetNumFrames():
        frame = thread.GetFrameAtIndex(lldb_frame_index)
        if frame.GetModule() and re.match(
            r"JIT\(.*?\)", frame.GetModule().GetFileSpec().GetFilename()
        ):
            # Compiled Java frame

            # Get function/filename/lineno from symbol context
            symbol = frame.GetSymbol()
            if not symbol:
                print("No symbol info for compiled Java frame: ", frame)
                sys.exit(1)
            line_entry = frame.GetLineEntry()
            prettified_frames.append(
                {
                    "function": symbol.GetName(),
                    "file": str(line_entry.GetFileSpec()) if line_entry else None,
                    "line": line_entry.GetLine() if line_entry else -1,
                }
            )

            # Skip art frames
            while True:
                art_stack_visitor = frame.EvaluateExpression(
                    """struct GetStackVisitor : public StackVisitor { GetStackVisitor(int depth_) : StackVisitor(Thread::Current(), NULL), depth(depth_) {} bool VisitFrame() { if (cur_depth_ == depth) { return false; } else { return true; } } int depth; }; GetStackVisitor visitor("""
                    + str(art_frame_index)
                    + """); visitor.WalkStack(true); visitor"""
                )
                art_method = frame.EvaluateExpression(
                    art_stack_visitor.GetName() + """.GetMethod()"""
                )
                if art_method.GetValueAsUnsigned() != 0:
                    art_method_name = frame.EvaluateExpression(
                        """art::PrettyMethod(""" + art_method.GetName() + """, true)"""
                    )
                    art_method_name_data = frame.EvaluateExpression(
                        art_method_name.GetName() + """.c_str()"""
                    ).GetValueAsUnsigned()
                    art_method_name_size = frame.EvaluateExpression(
                        art_method_name.GetName() + """.length()"""
                    ).GetValueAsUnsigned()
                    error = lldb.SBError()
                    art_method_name = process.ReadCStringFromMemory(
                        art_method_name_data, art_method_name_size + 1, error
                    )
                    if not error.Success:
                        print("Failed to read method name")
                        sys.exit(1)
                    if art_method_name != symbol.GetName():
                        print(
                            "Function names in native symbol and art runtime stack do not match: ",
                            symbol.GetName(),
                            " != ",
                            art_method_name,
                        )
                    art_frame_index = art_frame_index + 1
                    break
                art_frame_index = art_frame_index + 1

            # Skip native frames
            lldb_frame_index = lldb_frame_index + 1
            if lldb_frame_index < thread.GetNumFrames():
                frame = thread.GetFrameAtIndex(lldb_frame_index)
                if frame.GetModule() and re.match(
                    r"JIT\(.*?\)", frame.GetModule().GetFileSpec().GetFilename()
                ):
                    # Another compile Java frame
                    # Don't skip; leave it to the next iteration
                    continue
                elif frame.GetSymbol() and (
                    frame.GetSymbol().GetName() == "art_quick_invoke_stub"
                    or frame.GetSymbol().GetName() == "art_quick_invoke_static_stub"
                ):
                    # art_quick_invoke_stub / art_quick_invoke_static_stub
                    # Skip until we get past the next ArtMethod::Invoke()
                    while True:
                        lldb_frame_index = lldb_frame_index + 1
                        if lldb_frame_index >= thread.GetNumFrames():
                            print(
                                "ArtMethod::Invoke not found below art_quick_invoke_stub/art_quick_invoke_static_stub"
                            )
                            sys.exit(1)
                        frame = thread.GetFrameAtIndex(lldb_frame_index)
                        if (
                            frame.GetSymbol()
                            and frame.GetSymbol().GetName()
                            == "art::mirror::ArtMethod::Invoke(art::Thread*, unsigned int*, unsigned int, art::JValue*, char const*)"
                        ):
                            lldb_frame_index = lldb_frame_index + 1
                            break
                else:
                    print("Invalid frame below compiled Java frame: ", frame)
        elif (
            frame.GetSymbol()
            and frame.GetSymbol().GetName() == "art_quick_generic_jni_trampoline"
        ):
            # Interpreted JNI frame for x86_64

            # Skip art frames
            while True:
                art_stack_visitor = frame.EvaluateExpression(
                    """struct GetStackVisitor : public StackVisitor { GetStackVisitor(int depth_) : StackVisitor(Thread::Current(), NULL), depth(depth_) {} bool VisitFrame() { if (cur_depth_ == depth) { return false; } else { return true; } } int depth; }; GetStackVisitor visitor("""
                    + str(art_frame_index)
                    + """); visitor.WalkStack(true); visitor"""
                )
                art_method = frame.EvaluateExpression(
                    art_stack_visitor.GetName() + """.GetMethod()"""
                )
                if art_method.GetValueAsUnsigned() != 0:
                    # Get function/filename/lineno from ART runtime
                    art_method_name = frame.EvaluateExpression(
                        """art::PrettyMethod(""" + art_method.GetName() + """, true)"""
                    )
                    art_method_name_data = frame.EvaluateExpression(
                        art_method_name.GetName() + """.c_str()"""
                    ).GetValueAsUnsigned()
                    art_method_name_size = frame.EvaluateExpression(
                        art_method_name.GetName() + """.length()"""
                    ).GetValueAsUnsigned()
                    error = lldb.SBError()
                    function = process.ReadCStringFromMemory(
                        art_method_name_data, art_method_name_size + 1, error
                    )

                    prettified_frames.append(
                        {"function": function, "file": None, "line": -1}
                    )

                    art_frame_index = art_frame_index + 1
                    break
                art_frame_index = art_frame_index + 1

            # Skip native frames
            lldb_frame_index = lldb_frame_index + 1
            if lldb_frame_index < thread.GetNumFrames():
                frame = thread.GetFrameAtIndex(lldb_frame_index)
                if frame.GetSymbol() and (
                    frame.GetSymbol().GetName() == "art_quick_invoke_stub"
                    or frame.GetSymbol().GetName() == "art_quick_invoke_static_stub"
                ):
                    # art_quick_invoke_stub / art_quick_invoke_static_stub
                    # Skip until we get past the next ArtMethod::Invoke()
                    while True:
                        lldb_frame_index = lldb_frame_index + 1
                        if lldb_frame_index >= thread.GetNumFrames():
                            print(
                                "ArtMethod::Invoke not found below art_quick_invoke_stub/art_quick_invoke_static_stub"
                            )
                            sys.exit(1)
                        frame = thread.GetFrameAtIndex(lldb_frame_index)
                        if (
                            frame.GetSymbol()
                            and frame.GetSymbol().GetName()
                            == "art::mirror::ArtMethod::Invoke(art::Thread*, unsigned int*, unsigned int, art::JValue*, char const*)"
                        ):
                            lldb_frame_index = lldb_frame_index + 1
                            break
                else:
                    print("Invalid frame below compiled Java frame: ", frame)
        elif frame.GetSymbol() and re.search(
            r"art::interpreter::", frame.GetSymbol().GetName()
        ):
            # Interpreted Java frame

            while True:
                lldb_frame_index = lldb_frame_index + 1
                if lldb_frame_index >= thread.GetNumFrames():
                    print("art::interpreter::Execute not found in interpreter frame")
                    sys.exit(1)
                frame = thread.GetFrameAtIndex(lldb_frame_index)
                if (
                    frame.GetSymbol()
                    and frame.GetSymbol().GetName()
                    == "art::interpreter::Execute(art::Thread*, art::MethodHelper&, art::DexFile::CodeItem const*, art::ShadowFrame&, art::JValue)"
                ):
                    break

            # Skip art frames
            while True:
                art_stack_visitor = frame.EvaluateExpression(
                    """struct GetStackVisitor : public StackVisitor { GetStackVisitor(int depth_) : StackVisitor(Thread::Current(), NULL), depth(depth_) {} bool VisitFrame() { if (cur_depth_ == depth) { return false; } else { return true; } } int depth; }; GetStackVisitor visitor("""
                    + str(art_frame_index)
                    + """); visitor.WalkStack(true); visitor"""
                )
                art_method = frame.EvaluateExpression(
                    art_stack_visitor.GetName() + """.GetMethod()"""
                )
                if art_method.GetValueAsUnsigned() != 0:
                    # Get function/filename/lineno from ART runtime
                    art_method_name = frame.EvaluateExpression(
                        """art::PrettyMethod(""" + art_method.GetName() + """, true)"""
                    )
                    art_method_name_data = frame.EvaluateExpression(
                        art_method_name.GetName() + """.c_str()"""
                    ).GetValueAsUnsigned()
                    art_method_name_size = frame.EvaluateExpression(
                        art_method_name.GetName() + """.length()"""
                    ).GetValueAsUnsigned()
                    error = lldb.SBError()
                    function = process.ReadCStringFromMemory(
                        art_method_name_data, art_method_name_size + 1, error
                    )

                    line = frame.EvaluateExpression(
                        art_stack_visitor.GetName()
                        + """.GetMethod()->GetLineNumFromDexPC("""
                        + art_stack_visitor.GetName()
                        + """.GetDexPc(true))"""
                    ).GetValueAsUnsigned()

                    file_name = frame.EvaluateExpression(
                        art_method.GetName() + """->GetDeclaringClassSourceFile()"""
                    )
                    file_name_data = file_name.GetValueAsUnsigned()
                    file_name_size = frame.EvaluateExpression(
                        """(size_t)strlen(""" + file_name.GetName() + """)"""
                    ).GetValueAsUnsigned()
                    error = lldb.SBError()
                    file_name = process.ReadCStringFromMemory(
                        file_name_data, file_name_size + 1, error
                    )
                    if not error.Success():
                        print("Failed to read source file name")
                        sys.exit(1)

                    prettified_frames.append(
                        {"function": function, "file": file_name, "line": line}
                    )

                    art_frame_index = art_frame_index + 1
                    break
                art_frame_index = art_frame_index + 1

            # Skip native frames
            while True:
                lldb_frame_index = lldb_frame_index + 1
                if lldb_frame_index >= thread.GetNumFrames():
                    print("Can not get past interpreter native frames")
                    sys.exit(1)
                frame = thread.GetFrameAtIndex(lldb_frame_index)
                if frame.GetSymbol() and not re.search(
                    r"art::interpreter::", frame.GetSymbol().GetName()
                ):
                    break
        else:
            # Other frames. Add them as-is.
            frame = thread.GetFrameAtIndex(lldb_frame_index)
            lldb_frame_index = lldb_frame_index + 1
            if frame.GetModule():
                module_name = frame.GetModule().GetFileSpec().GetFilename()
                if not module_name in [
                    "libartd.so",
                    "dalvikvm32",
                    "dalvikvm64",
                    "libc.so.6",
                ]:
                    prettified_frames.append(
                        {
                            "function": frame.GetSymbol().GetName()
                            if frame.GetSymbol()
                            else None,
                            "file": str(frame.GetLineEntry().GetFileSpec())
                            if frame.GetLineEntry()
                            else None,
                            "line": frame.GetLineEntry().GetLine()
                            if frame.GetLineEntry()
                            else -1,
                        }
                    )

    for prettified_frame in prettified_frames:
        print(
            prettified_frame["function"],
            prettified_frame["file"],
            prettified_frame["line"],
        )


def __lldb_init_module(debugger, internal_dict):
    debugger.HandleCommand("command script add -f host_art_bt.host_art_bt host_art_bt")



"""Test that hidden ivars in a shared library are visible from the main executable."""

import subprocess

import unittest
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class HiddenIvarsTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.source = "main.m"
        self.line = line_number(self.source, "// breakpoint1")
        # The makefile names of the shared libraries as they appear in DYLIB_NAME.
        # The names should have no loading "lib" or extension as they will be
        # localized
        self.shlib_names = ["InternalDefiner"]

    @skipIf(
        debug_info=no_match("dsym"),
        bugnumber="This test requires a stripped binary and a dSYM",
    )
    def test_expr_stripped(self):
        if self.getArchitecture() == "i386":
            self.skipTest("requires modern objc runtime")
        else:
            self.build()
            self.expr(True)

    def test_expr(self):
        if self.getArchitecture() == "i386":
            self.skipTest("requires modern objc runtime")
        else:
            self.build()
            self.expr(False)

    @skipIf(
        debug_info=no_match("dsym"),
        bugnumber="This test requires a stripped binary and a dSYM",
    )
    def test_frame_variable_stripped(self):
        if self.getArchitecture() == "i386":
            self.skipTest("requires modern objc runtime")
        else:
            self.build()
            self.frame_var(True)

    def test_frame_variable(self):
        if self.getArchitecture() == "i386":
            self.skipTest("requires modern objc runtime")
        else:
            self.build()
            self.frame_var(False)

    @unittest.expectedFailure  # rdar://18683637
    def test_frame_variable_across_modules(self):
        if self.getArchitecture() == "i386":
            self.skipTest("requires modern objc runtime")
        else:
            self.build()
            self.common_setup(False)
            self.expect(
                "frame variable k->bar", VARIABLES_DISPLAYED_CORRECTLY, substrs=["= 3"]
            )

    def common_setup(self, strip):
        if strip:
            exe = self.getBuildArtifact("stripped/a.out")
        else:
            exe = self.getBuildArtifact("a.out")
        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Create the breakpoint inside function 'main'.
        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Register our shared libraries for remote targets so they get
        # automatically uploaded
        environment = self.registerSharedLibrariesWithTarget(target, self.shlib_names)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            None, environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break inside the foo function which takes a bar_ptr argument.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

    def expr(self, strip):
        self.common_setup(strip)

        # This should display correctly.
        self.expect(
            "expression (j->_definer->foo)",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["= 4"],
        )

        self.expect(
            "expression (j->_definer->bar)",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["= 5"],
        )

        if strip:
            self.expect(
                "expression *(j->_definer)",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=["foo = 4"],
            )
        else:
            self.expect(
                "expression *(j->_definer)",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=["foo = 4", "bar = 5"],
            )

        self.expect(
            "expression (k->foo)", VARIABLES_DISPLAYED_CORRECTLY, substrs=["= 2"]
        )

        self.expect(
            "expression (k->bar)", VARIABLES_DISPLAYED_CORRECTLY, substrs=["= 3"]
        )

        self.expect(
            "expression k.filteredDataSource",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=[" = 0x", '"2 elements"'],
        )

        if strip:
            self.expect(
                "expression *(k)",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=["foo = 2", " = 0x", '"2 elements"'],
            )
        else:
            self.expect(
                "expression *(k)",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=[
                    "foo = 2",
                    "bar = 3",
                    "_filteredDataSource = 0x",
                    '"2 elements"',
                ],
            )

    def frame_var(self, strip):
        self.common_setup(strip)

        # This should display correctly.
        self.expect(
            "frame variable j->_definer->foo",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["= 4"],
        )

        if not strip:
            self.expect(
                "frame variable j->_definer->bar",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=["= 5"],
            )

        if strip:
            self.expect(
                "frame variable *j->_definer",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=["foo = 4"],
            )
        else:
            self.expect(
                "frame variable *j->_definer",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=["foo = 4", "bar = 5"],
            )

        self.expect(
            "frame variable k->foo", VARIABLES_DISPLAYED_CORRECTLY, substrs=["= 2"]
        )

        self.expect(
            "frame variable k->_filteredDataSource",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=[" = 0x", '"2 elements"'],
        )

        if strip:
            self.expect(
                "frame variable *k",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=["foo = 2", "_filteredDataSource = 0x", '"2 elements"'],
            )
        else:
            self.expect(
                "frame variable *k",
                VARIABLES_DISPLAYED_CORRECTLY,
                substrs=[
                    "foo = 2",
                    "bar = 3",
                    "_filteredDataSource = 0x",
                    '"2 elements"',
                ],
            )



"""
Test that re-running a process from within the same target
after rebuilding the executable flushes the scratch TypeSystems
tied to that process.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


class TestRerunExpr(TestBase):
    SHARED_BUILD_TESTCASE = False

    # FIXME: on Windows rebuilding the binary isn't enough to unload it
    #        on progrem restart. One will have to try hard to evict
    #        the module from the ModuleList (possibly including a call to
    #        SBDebugger::MemoryPressureDetected.
    @skipIfWindows
    def test(self):
        """
        Tests whether re-launching a process without destroying
        the owning target keeps invalid ASTContexts in the
        scratch AST's importer.

        We test this by:
        1. Evaluating an expression to import 'struct Foo' into
           the scratch AST
        2. Change the definition of 'struct Foo' and rebuild the executable
        3. Re-launch the process
        4. Evaluate the same expression in (1). We expect to have only
           the latest definition of 'struct Foo' in the scratch AST.
        """
        self.build(dictionary={"CXX_SOURCES": "main.cpp", "EXE": "a.out"})

        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        target.BreakpointCreateBySourceRegex("return", lldb.SBFileSpec("rebuild.cpp"))
        target.BreakpointCreateBySourceRegex("return", lldb.SBFileSpec("main.cpp"))
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.expect_expr(
            "foo",
            result_type="Foo",
            result_children=[ValueCheck(name="m_val", value="42")],
        )

        # Delete the executable to force make to rebuild it.
        remove_file(exe)
        self.build(dictionary={"CXX_SOURCES": "rebuild.cpp", "EXE": "a.out"})

        # Rerun program within the same target
        process.Destroy()
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.expect_expr(
            "foo",
            result_type="Foo",
            result_children=[
                ValueCheck(
                    name="Base", children=[ValueCheck(name="m_base_val", value="42")]
                ),
                ValueCheck(name="m_derived_val", value="137"),
            ],
        )

        self.filecheck("target module dump ast", __file__)

        # The new definition 'struct Foo' is in the scratch AST
        # CHECK:      |-CXXRecordDecl {{.*}} struct Foo definition
        # CHECK:      | |-public 'Base'
        # CHECK-NEXT: | `-FieldDecl {{.*}} m_derived_val 'int'
        # CHECK-NEXT: `-CXXRecordDecl {{.*}} struct Base definition
        # CHECK:        `-FieldDecl {{.*}} m_base_val 'int'

        # ...but the original definition of 'struct Foo' is not in the scratch AST anymore
        # CHECK-NOT: FieldDecl {{.*}} m_val 'int'



"""
Test typedef types.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test import decorators


class TestCppTypedef(TestBase):
    def test_typedef(self):
        """
        Test that we retrieve typedefed types correctly
        """

        self.build()
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        # First of all, check that we can get a typedefed type correctly in a simple case.
        expr_result = self.expect_expr(
            "(GlobalTypedef)s",
            result_type="GlobalTypedef",
            result_children=[ValueCheck(value="0.5")],
        )

        # The type should be a typedef.
        typedef_type = expr_result.GetType()
        self.assertTrue(typedef_type.IsValid())
        self.assertTrue(typedef_type.IsTypedefType())

        # The underlying type should be S<float>.
        typedefed_type = typedef_type.GetTypedefedType()
        self.assertTrue(typedefed_type.IsValid())
        self.assertEqual(typedefed_type.GetName(), "S<float>")

        # Check that we can get a typedefed type correctly in the case
        # when an elaborated type is created during the parsing
        expr_result = self.expect_expr(
            "(GlobalTypedef::V)s.value", result_type="GlobalTypedef::V"
        )

        # The type should be a typedef.
        typedef_type = expr_result.GetType()
        self.assertTrue(typedef_type.IsValid())
        self.assertTrue(typedef_type.IsTypedefType())

        # The underlying type should be float.
        typedefed_type = typedef_type.GetTypedefedType()
        self.assertTrue(typedefed_type.IsValid())
        self.assertEqual(typedefed_type.GetName(), "float")

        # Try accessing a typedef inside a namespace.
        self.expect_expr(
            "(ns::NamespaceTypedef)s", result_children=[ValueCheck(value="0.5")]
        )

        # Try accessing a typedef inside a struct/class.
        # FIXME: This doesn't actually work. StructTypedef just gets injected
        # by the local variable in the expression evaluation context.
        self.expect_expr(
            "(ST::StructTypedef)s", result_children=[ValueCheck(value="0.5")]
        )
        # This doesn't work for the reason above. There is no local variable
        # injecting OtherStructTypedef so we will actually error here.
        self.expect(
            "expression -- (NonLocalVarStruct::OtherStructTypedef)1",
            error=True,
            substrs=["no member named 'OtherStructTypedef' in 'NonLocalVarStruct'"],
        )

        # Check the generated Clang AST.
        self.filecheck("image dump ast a.out", __file__, "--strict-whitespace")


# CHECK:      {{^}}|-TypedefDecl {{.*}} GlobalTypedef 'S<float>'
# CHECK:      {{^}}|-NamespaceDecl {{.*}} ns
# CHECK-NEXT: {{^}}| `-TypedefDecl {{.*}} NamespaceTypedef 'S<float>'
# CHECK:      {{^}}|-CXXRecordDecl {{.*}} struct ST definition
# CHECK:      {{^}}| `-TypedefDecl {{.*}} StructTypedef 'S<float>'



"""
Test basic std::unique_ptr functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestUniquePtr(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    @skipIf(macos_version=["<", "15.0"])
    @skipIfLinux  # s.reset() causes link errors on ubuntu 18.04/Clang 9
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
            [">", "16.0"]
        ):
            ptr_type = "std::unique_ptr<int>"
        else:
            ptr_type = "std::unique_ptr<int, std::default_delete<int> >"

        self.expect_expr(
            "s",
            result_type=ptr_type,
            result_summary="3",
            result_children=[ValueCheck(name="pointer")],
        )
        self.expect_expr("*s", result_type="int", result_value="3")
        self.expect_expr("*s = 5", result_type="int", result_value="5")
        self.expect_expr("*s", result_type="int", result_value="5")
        self.expect_expr("(bool)s", result_type="bool", result_value="true")
        self.expect("expr s.reset()")
        self.expect_expr("(bool)s", result_type="bool", result_value="false")



"""
Test quoting of arguments to lldb commands.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SettingsCommandTestCase(TestBase):
    output_file_name = "output.txt"

    @classmethod
    def classCleanup(cls):
        """Cleanup the test byproducts."""
        cls.RemoveTempFile(SettingsCommandTestCase.output_file_name)

    @no_debug_info_test
    def test(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # No quotes.
        self.expect_args("a b c", "a\0b\0c\0")
        # Single quotes.
        self.expect_args("'a b c'", "a b c\0")
        # Double quotes.
        self.expect_args('"a b c"', "a b c\0")
        # Single quote escape.
        self.expect_args("'a b\\' c", "a b\\\0c\0")
        # Double quote escape.
        self.expect_args('"a b\\" c"', 'a b" c\0')
        self.expect_args('"a b\\\\" c', "a b\\\0c\0")
        # Single quote in double quotes.
        self.expect_args('"a\'b"', "a'b\0")
        # Double quotes in single quote.
        self.expect_args("'a\"b'", 'a"b\0')
        # Combined quotes.
        self.expect_args("\"a b\"c'd e'", "a bcd e\0")
        # Bare single/double quotes.
        self.expect_args("a\\'b", "a'b\0")
        self.expect_args('a\\"b', 'a"b\0')

    def expect_args(self, args_in, args_out):
        """Test argument parsing. Run the program with args_in. The program dumps its arguments
        to stdout. Compare the stdout with args_out."""

        filename = SettingsCommandTestCase.output_file_name
        outfile = self.getBuildArtifact(filename)

        if lldb.remote_platform:
            outfile_arg = lldbutil.append_to_process_working_directory(self, filename)
        else:
            outfile_arg = outfile

        self.runCmd("process launch -- %s %s" % (outfile_arg, args_in))

        if lldb.remote_platform:
            src_file_spec = lldb.SBFileSpec(outfile_arg, False)
            dst_file_spec = lldb.SBFileSpec(outfile, True)
            lldb.remote_platform.Get(src_file_spec, dst_file_spec)

        with open(outfile, "r") as f:
            output = f.read()

        self.RemoveTempFile(outfile)
        self.assertEqual(output, args_out)



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetSize()
    obj.GetInstructionAtIndex(0xFFFFFFFF)
    obj.AppendInstruction(lldb.SBInstruction())
    try:
        obj.Print(None)
    except Exception:
        pass
    obj.GetDescription(lldb.SBStream())
    obj.DumpEmulationForAllInstructions("armv7")
    obj.Clear()
    for inst in obj:
        s = str(inst)



import gdbremote_testcase
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestGdbRemoteSingleStep(gdbremote_testcase.GdbRemoteTestCaseBase):
    @skipIfWindows  # No pty support to test any inferior std -i/e/o
    @skipIf(triple="^mips")
    def test_single_step_only_steps_one_instruction_with_s(self):
        self.build()
        self.set_inferior_startup_launch()
        self.single_step_only_steps_one_instruction(
            use_Hc_packet=True, step_instruction="s"
        )



"""
This tests some simple examples of parsing regex commands
"""

import os
import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestCommandRegexParsing(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_sample_rename_this(self):
        """Try out some simple regex commands, make sure they parse correctly."""
        self.runCmd(
            "command regex one-substitution 's/(.+)/echo-cmd %1-first %1-second %1-third/'"
        )
        self.expect(
            "one-substitution ASTRING",
            substrs=["ASTRING-first", "ASTRING-second", "ASTRING-third"],
        )

        self.runCmd(
            "command regex two-substitution 's/([^ ]+) ([^ ]+)/echo-cmd %1-first %2-second %1-third %2-fourth/'"
        )
        self.expect(
            "two-substitution ASTRING BSTRING",
            substrs=[
                "ASTRING-first",
                "BSTRING-second",
                "ASTRING-third",
                "BSTRING-fourth",
            ],
        )

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.runCmd(
            "command script import "
            + os.path.join(self.getSourceDir(), "echo_command.py")
        )
        self.runCmd("command script add echo-cmd -f echo_command.echo_command")



"""
Test that we properly print multiple types.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test import decorators


class TestTypeLookupAnonStruct(TestBase):
    def test_lookup_anon_struct(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.expect_var_path("unnamed_derived.y", value="2")
        self.expect_var_path("unnamed_derived.z", value="13")
        self.expect(
            'frame variable "derb.x"',
            error=True,
            substrs=['"x" is not a member of "(DerivedB) derb"']
        )
        self.expect(
            'frame variable "derb.y"',
            error=True,
            substrs=['"y" is not a member of "(DerivedB) derb"']
        )
        self.expect_var_path("derb.w", value="14")
        self.expect_var_path("derb.k", value="15")
        self.expect_var_path("derb.a.x", value="1")
        self.expect_var_path("derb.a.y", value="2")

        self.expect_var_path("multi1.m", value="16")
        self.expect_var_path("multi1.y", value="30")

        self.expect_var_path("multi2.i", value="42")
        self.expect_var_path("multi2.w", value="23")
        self.expect_var_path("multi2.a.x", value="1")
        self.expect_var_path("multi2.a.y", value="2")
        self.expect_var_path("multi2.y", value="2")
        self.expect_var_path("multi2.n", value="7")



"""
Test the iteration protocol for some lldb container objects.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LLDBIteratorTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break inside main().
        self.line1 = line_number("main.cpp", "// Set break point at this line.")
        self.line2 = line_number("main.cpp", "// And that line.")

    def test_lldb_iter_module(self):
        """Test module_iter works correctly for SBTarget -> SBModule."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        breakpoint = target.BreakpointCreateByLocation("main.cpp", self.line1)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if not process:
            self.fail("SBTarget.LaunchProcess() failed")

        from lldbsuite.test.lldbutil import get_description

        yours = []
        for i in range(target.GetNumModules()):
            yours.append(target.GetModuleAtIndex(i))
        mine = []
        for m in target.module_iter():
            mine.append(m)

        self.assertEqual(len(yours), len(mine))
        for i in range(len(yours)):
            if self.TraceOn():
                print("yours[%d]='%s'" % (i, get_description(yours[i])))
                print("mine[%d]='%s'" % (i, get_description(mine[i])))
            self.assertEqual(
                yours[i],
                mine[i],
                "UUID+FileSpec of yours[{0}] and mine[{0}] matches".format(i),
            )

    def test_lldb_iter_breakpoint(self):
        """Test breakpoint_iter works correctly for SBTarget -> SBBreakpoint."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        breakpoint = target.BreakpointCreateByLocation("main.cpp", self.line1)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)
        breakpoint = target.BreakpointCreateByLocation("main.cpp", self.line2)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        self.assertEqual(target.GetNumBreakpoints(), 2)

        from lldbsuite.test.lldbutil import get_description

        yours = []
        for i in range(target.GetNumBreakpoints()):
            yours.append(target.GetBreakpointAtIndex(i))
        mine = []
        for b in target.breakpoint_iter():
            mine.append(b)

        self.assertEqual(len(yours), len(mine))
        for i in range(len(yours)):
            if self.TraceOn():
                print("yours[%d]='%s'" % (i, get_description(yours[i])))
                print("mine[%d]='%s'" % (i, get_description(mine[i])))
            self.assertEqual(
                yours[i], mine[i], "ID of yours[{0}] and mine[{0}] matches".format(i)
            )

    @skipIfWindows  # This test is flaky on Windows
    def test_lldb_iter_frame(self):
        """Test iterator works correctly for SBProcess->SBThread->SBFrame."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        breakpoint = target.BreakpointCreateByLocation("main.cpp", self.line1)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if not process:
            self.fail("SBTarget.LaunchProcess() failed")

        from lldbsuite.test.lldbutil import print_stacktrace

        stopped_due_to_breakpoint = False
        for thread in process:
            if self.TraceOn():
                print_stacktrace(thread)
            ID = thread.GetThreadID()
            if thread.GetStopReason() == lldb.eStopReasonBreakpoint:
                stopped_due_to_breakpoint = True
            for frame in thread:
                self.assertEqual(frame.GetThread().GetThreadID(), ID)
                if self.TraceOn():
                    print(frame)

        self.assertTrue(stopped_due_to_breakpoint)



"""
Test that a Python breakpoint callback defined in another Python
breakpoint callback works properly. 
"""


import lldb
import os
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestNestedBreakpointCommands(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_nested_commands(self):
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.callback_module = "make_bkpt_cmds"
        self.do_test()

    def do_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        outer_bkpt = target.BreakpointCreateBySourceRegex(
            "Set outer breakpoint here", self.main_source_file
        )
        cmd_file_path = os.path.join(self.getSourceDir(), f"{self.callback_module}.py")
        self.runCmd(f"command script import {cmd_file_path}")
        outer_bkpt.SetScriptCallbackFunction(f"{self.callback_module}.outer_callback")

        process.Continue()

        self.assertEqual(
            thread.stop_reason, lldb.eStopReasonBreakpoint, "Right stop reason"
        )

        bkpt_no = thread.stop_reason_data[0]

        # We made the callbacks record the new breakpoint ID and the number of
        # times a callback ran in some globals in the target.  Find them now:
        exec_module = target.FindModule(target.executable)
        self.assertTrue(exec_module.IsValid(), "Found executable module")
        var = exec_module.FindFirstGlobalVariable(target, "g_global")
        self.assertSuccess(var.GetError(), "Found globals")
        num_hits = var.GetChildAtIndex(1).GetValueAsUnsigned()
        inner_id = var.GetChildAtIndex(2).GetValueAsUnsigned()

        # Make sure they have the right values:
        self.assertEqual(bkpt_no, inner_id, "Hit the right breakpoint")
        self.assertEqual(num_hits, 2, "Right counter end value")
        self.assertEqual(thread.frames[0].name, "main", "Got to main")

        self.assertEqual(outer_bkpt.GetHitCount(), 1, "Hit outer breakpoint once")

        inner_bkpt = target.FindBreakpointByID(inner_id)
        self.assertEqual(inner_bkpt.GetHitCount(), 1, "Hit inner breakpoint once")



"""Test the SBProgress API."""

import lldb
from lldbsuite.test.lldbtest import *


class SBProgressTestCase(TestBase):
    def test_with_external_bit_set(self):
        """Test SBProgress can handle null events."""

        progress = lldb.SBProgress("Test SBProgress", "Test progress", 3, self.dbg)
        listener = lldb.SBListener("Test listener")
        broadcaster = self.dbg.GetBroadcaster()
        broadcaster.AddListener(listener, lldb.eBroadcastBitExternalProgress)
        event = lldb.SBEvent()
        # Sample JSON we're expecting:
        # { id = 2, title = "Test SBProgress", details = "Test progress", type = update, progress = 1 of 3}
        # details remains the same as specified in the constructor of the progress
        # until we update it in the increment function, so we check for the Null and empty string case
        # that details hasn't changed, but progress x of 3 has.
        progress.Increment(1, None)
        self.assertTrue(listener.GetNextEvent(event))
        stream = lldb.SBStream()
        event.GetDescription(stream)
        self.assertIn("Test progress", stream.GetData())
        self.assertIn("1 of 3", stream.GetData())

        progress.Increment(1, "")
        self.assertTrue(listener.GetNextEvent(event))
        event.GetDescription(stream)
        self.assertIn("Test progress", stream.GetData())
        self.assertIn("2 of 3", stream.GetData())

        progress.Increment(1, "Step 3")
        self.assertTrue(listener.GetNextEvent(event))
        stream = lldb.SBStream()
        event.GetDescription(stream)
        self.assertIn("Step 3", stream.GetData())

    def test_progress_finalize_non_deterministic_progress(self):
        """Test SBProgress finalize sends the progressEnd event"""

        progress = lldb.SBProgress("Test SBProgress", "Test finalize", self.dbg)
        listener = lldb.SBListener("Test listener")
        broadcaster = self.dbg.GetBroadcaster()
        broadcaster.AddListener(listener, lldb.eBroadcastBitExternalProgress)
        event = lldb.SBEvent()
        progress.Finalize()
        self.assertTrue(listener.WaitForEvent(5, event))
        stream = lldb.SBStream()
        event.GetDescription(stream)
        self.assertIn("type = end", stream.GetData())

    def test_progress_finalize_deterministic_progress(self):
        """Test SBProgress finalize sends the progressEnd event"""

        progress = lldb.SBProgress("Test SBProgress", "Test finalize", 13, self.dbg)
        listener = lldb.SBListener("Test listener")
        broadcaster = self.dbg.GetBroadcaster()
        broadcaster.AddListener(listener, lldb.eBroadcastBitExternalProgress)
        event = lldb.SBEvent()
        progress.Finalize()
        self.assertTrue(listener.WaitForEvent(5, event))
        stream = lldb.SBStream()
        event.GetDescription(stream)
        # Note even for progresses with a total, the total isn't
        # sent in the end message.
        self.assertIn("type = end", stream.GetData())



"""
Test that single thread step over deadlock issue can be resolved 
after timeout.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SingleThreadStepTimeoutTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.cpp"
        self.build()

    def verify_hit_correct_line(self, pattern):
        target_line = line_number(self.main_source, pattern)
        self.assertNotEqual(target_line, 0, "Could not find source pattern " + pattern)
        cur_line = self.thread.frames[0].GetLineEntry().GetLine()
        self.assertEqual(
            cur_line,
            target_line,
            "Stepped to line %d instead of expected %d with pattern '%s'."
            % (cur_line, target_line, pattern),
        )

    def step_over_deadlock_helper(self):
        (target, _, self.thread, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint1 here", lldb.SBFileSpec(self.main_source)
        )

        signal_main_thread_value = target.FindFirstGlobalVariable("signal_main_thread")
        self.assertTrue(signal_main_thread_value.IsValid())

        # Change signal_main_thread global variable to 1 so that worker thread loop can
        # terminate and move forward to signal main thread
        signal_main_thread_value.SetValueFromCString("1")

        self.thread.StepOver(lldb.eOnlyThisThread)
        self.verify_hit_correct_line("// Finish step-over from breakpoint1")

    @skipIfWindows
    def test_step_over_deadlock_small_timeout_fast_stepping(self):
        """Test single thread step over deadlock on other threads can be resolved after timeout with small timeout and fast stepping."""
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 10"
        )
        self.dbg.HandleCommand("settings set target.use-fast-stepping true")
        self.step_over_deadlock_helper()

    @skipIfWindows
    def test_step_over_deadlock_small_timeout_slow_stepping(self):
        """Test single thread step over deadlock on other threads can be resolved after timeout with small timeout and slow stepping."""
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 10"
        )
        self.dbg.HandleCommand("settings set target.use-fast-stepping false")
        self.step_over_deadlock_helper()

    @skipIfWindows
    def test_step_over_deadlock_large_timeout_fast_stepping(self):
        """Test single thread step over deadlock on other threads can be resolved after timeout with large timeout and fast stepping."""
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 2000"
        )
        self.dbg.HandleCommand("settings set target.use-fast-stepping true")
        self.step_over_deadlock_helper()

    @skipIfWindows
    def test_step_over_deadlock_large_timeout_slow_stepping(self):
        """Test single thread step over deadlock on other threads can be resolved after timeout with large timeout and slow stepping."""
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 2000"
        )
        self.dbg.HandleCommand("settings set target.use-fast-stepping false")
        self.step_over_deadlock_helper()

    def step_over_multi_calls_helper(self):
        (target, _, self.thread, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint2 here", lldb.SBFileSpec(self.main_source)
        )
        self.thread.StepOver(lldb.eOnlyThisThread)
        self.verify_hit_correct_line("// Finish step-over from breakpoint2")

    @skipIfWindows
    def test_step_over_multi_calls_small_timeout_fast_stepping(self):
        """Test step over source line with multiple call instructions works fine with small timeout and fast stepping."""
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 10"
        )
        self.dbg.HandleCommand("settings set target.use-fast-stepping true")
        self.step_over_multi_calls_helper()

    @skipIfWindows
    def test_step_over_multi_calls_small_timeout_slow_stepping(self):
        """Test step over source line with multiple call instructions works fine with small timeout and slow stepping."""
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 10"
        )
        self.dbg.HandleCommand("settings set target.use-fast-stepping false")
        self.step_over_multi_calls_helper()

    @skipIfWindows
    def test_step_over_multi_calls_large_timeout_fast_stepping(self):
        """Test step over source line with multiple call instructions works fine with large timeout and fast stepping."""
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 2000"
        )
        self.dbg.HandleCommand("settings set target.use-fast-stepping true")
        self.step_over_multi_calls_helper()

    @skipIfWindows
    def test_step_over_multi_calls_large_timeout_slow_stepping(self):
        """Test step over source line with multiple call instructions works fine with large timeout and slow stepping."""
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 2000"
        )
        self.dbg.HandleCommand("settings set target.use-fast-stepping false")
        self.step_over_multi_calls_helper()

    @skipIfWindows
    def test_step_over_deadlock_with_inner_breakpoint_continue(self):
        """Test step over deadlock function with inner breakpoint will trigger the breakpoint
        and later continue will finish the stepping.
        """
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 2000"
        )
        (target, process, self.thread, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint1 here", lldb.SBFileSpec(self.main_source)
        )

        signal_main_thread_value = target.FindFirstGlobalVariable("signal_main_thread")
        self.assertTrue(signal_main_thread_value.IsValid())

        # Change signal_main_thread global variable to 1 so that worker thread loop can
        # terminate and move forward to signal main thread
        signal_main_thread_value.SetValueFromCString("1")

        # Set breakpoint on inner function call
        inner_breakpoint = target.BreakpointCreateByLocation(
            lldb.SBFileSpec(self.main_source),
            line_number("main.cpp", "// Set interrupt breakpoint here"),
            0,
            0,
            lldb.SBFileSpecList(),
            False,
        )

        # Step over will hit the inner breakpoint and stop
        self.thread.StepOver(lldb.eOnlyThisThread)
        self.assertStopReason(self.thread.GetStopReason(), lldb.eStopReasonBreakpoint)
        thread1 = lldbutil.get_one_thread_stopped_at_breakpoint(
            process, inner_breakpoint
        )
        self.assertTrue(
            thread1.IsValid(),
            "We are indeed stopped at inner breakpoint inside deadlock_func",
        )

        # Continue the process should complete the step-over
        process.Continue()
        self.assertState(process.GetState(), lldb.eStateStopped)
        self.assertStopReason(self.thread.GetStopReason(), lldb.eStopReasonPlanComplete)

        self.verify_hit_correct_line("// Finish step-over from breakpoint1")

    @skipIfWindows
    def test_step_over_deadlock_with_inner_breakpoint_step(self):
        """Test step over deadlock function with inner breakpoint will trigger the breakpoint
        and later step still works
        """
        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 2000"
        )
        (target, process, self.thread, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint1 here", lldb.SBFileSpec(self.main_source)
        )

        signal_main_thread_value = target.FindFirstGlobalVariable("signal_main_thread")
        self.assertTrue(signal_main_thread_value.IsValid())

        # Change signal_main_thread global variable to 1 so that worker thread loop can
        # terminate and move forward to signal main thread
        signal_main_thread_value.SetValueFromCString("1")

        # Set breakpoint on inner function call
        inner_breakpoint = target.BreakpointCreateByLocation(
            lldb.SBFileSpec(self.main_source),
            line_number("main.cpp", "// Set interrupt breakpoint here"),
            0,
            0,
            lldb.SBFileSpecList(),
            False,
        )

        # Step over will hit the inner breakpoint and stop
        self.thread.StepOver(lldb.eOnlyThisThread)
        self.assertStopReason(self.thread.GetStopReason(), lldb.eStopReasonBreakpoint)
        thread1 = lldbutil.get_one_thread_stopped_at_breakpoint(
            process, inner_breakpoint
        )
        self.assertTrue(
            thread1.IsValid(),
            "We are indeed stopped at inner breakpoint inside deadlock_func",
        )

        # Step still works
        self.thread.StepOver(lldb.eOnlyThisThread)
        self.assertState(process.GetState(), lldb.eStateStopped)
        self.assertStopReason(self.thread.GetStopReason(), lldb.eStopReasonPlanComplete)

        self.verify_hit_correct_line("// Finish step-over from inner breakpoint")

    @skipIfWindows
    def test_step_over_deadlock_with_user_async_interrupt(self):
        """Test step over deadlock function with large timeout then send async interrupt
        should report correct stop reason
        """

        self.dbg.HandleCommand(
            "settings set target.process.thread.single-thread-plan-timeout 2000000"
        )

        (target, process, self.thread, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint1 here", lldb.SBFileSpec(self.main_source)
        )

        signal_main_thread_value = target.FindFirstGlobalVariable("signal_main_thread")
        self.assertTrue(signal_main_thread_value.IsValid())

        # Change signal_main_thread global variable to 1 so that worker thread loop can
        # terminate and move forward to signal main thread
        signal_main_thread_value.SetValueFromCString("1")

        self.dbg.SetAsync(True)

        # This stepping should block due to large timeout and should be interrupted by the
        # async interrupt from the worker thread
        self.thread.StepOver(lldb.eOnlyThisThread)
        time.sleep(1)

        listener = self.dbg.GetListener()
        lldbutil.expect_state_changes(self, listener, process, [lldb.eStateRunning])
        self.dbg.SetAsync(False)

        process.SendAsyncInterrupt()

        lldbutil.expect_state_changes(self, listener, process, [lldb.eStateStopped])
        self.assertStopReason(self.thread.GetStopReason(), lldb.eStopReasonSignal)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        (_, process, _, _) = lldbutil.run_to_source_breakpoint(
            self, "Break: const_method begin", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr("bar()", result_value="2", result_type="int")
        self.expect(
            "expression m_const_mem = 2.0",
            error=True,
            substrs=[
                "note: Possibly trying to mutate object in a const context. Try running the expression with",
            ],
            matching=False,
        )
        self.expect(
            "expression m_mem = 2.0",
            error=True,
            substrs=[
                "cannot assign to non-static data member within const member function",
                "note: Possibly trying to mutate object in a const context. Try running the expression with",
            ],
        )
        self.expect_expr("m_mem", result_value="-2")

        # Test short and long --c++-ignore-context-qualifiers option.
        self.expect(
            "expression --c++-ignore-context-qualifiers -- m_mem = 3.0",
            error=False,
        )
        self.expect_expr("m_mem", result_value="3")

        self.expect(
            "expression -Q -- m_mem = 4.0",
            error=False,
        )
        self.expect_expr("m_mem", result_value="4")

        # Test --c++-ignore-context-qualifiers via SBExpressionOptions.
        options = lldb.SBExpressionOptions()
        options.SetBooleanLanguageOption("c++-ignore-context-qualifiers", True)
        self.expect_expr("m_mem = -2.0; m_mem", options=options, result_value="-2")

        self.expect_expr("((Foo*)this)->bar()", result_type="double", result_value="5")

        lldbutil.continue_to_source_breakpoint(
            self,
            process,
            "Break: const_method no-this lambda",
            lldb.SBFileSpec("main.cpp"),
        )

        self.expect(
            "expression x = 7.0",
            error=True,
            substrs=[
                "cannot assign to non-static data member within const member function",
                "note: Possibly trying to mutate object in a const context. Try running the expression with",
            ],
        )
        self.expect_expr("x", result_value="2")

        self.expect_expr("x = -5; x", options=options, result_value="-5")

        lldbutil.continue_to_source_breakpoint(
            self,
            process,
            "Break: const_method mutable no-this lambda",
            lldb.SBFileSpec("main.cpp"),
        )

        self.expect_expr("x = 7.0; x", result_value="7")

        lldbutil.continue_to_source_breakpoint(
            self, process, "Break: const_method lambda", lldb.SBFileSpec("main.cpp")
        )

        # FIXME: mutating this capture should be disallowed in a non-mutable lambda.
        self.expect_expr("y = 8.0")
        self.expect_expr("bar()", result_value="2", result_type="int")
        self.expect(
            "expression m_const_mem = 2.0",
            error=True,
            substrs=[
                "note: Possibly trying to mutate object in a const context. Try running the expression with",
            ],
            matching=False,
        )
        self.expect(
            "expression m_mem = 2.0",
            error=True,
            substrs=[
                "cannot assign to non-static data member within const member function",
                "note: Possibly trying to mutate object in a const context. Try running the expression with",
            ],
        )
        self.expect_expr("m_mem", result_value="-2")

        self.expect_expr("m_mem = -1; m_mem", options=options, result_value="-1")

        self.expect_expr("((Foo*)this)->bar()", result_type="double", result_value="5")

        lldbutil.continue_to_source_breakpoint(
            self,
            process,
            "Break: const_method mutable lambda",
            lldb.SBFileSpec("main.cpp"),
        )

        self.expect_expr("y = 9.0")
        self.expect_expr("bar()", result_value="2", result_type="int")
        self.expect(
            "expression m_const_mem = 2.0",
            error=True,
            substrs=[
                "note: Possibly trying to mutate object in a const context. Try running the expression with",
            ],
            matching=False,
        )
        self.expect(
            "expression m_mem = 2.0",
            error=True,
            substrs=[
                "cannot assign to non-static data member within const member function",
                "note: Possibly trying to mutate object in a const context. Try running the expression with",
            ],
        )
        self.expect_expr("m_mem", result_value="-1")

        self.expect_expr("m_mem = -2; m_mem", options=options, result_value="-2")



"""
Test lldb Python commands.
"""


import sys
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class CmdPythonTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.build()
        self.pycmd_tests()

    def pycmd_tests(self):
        self.runCmd("command source py_import")

        # Test that we did indeed add these commands as user commands:
        interp = self.dbg.GetCommandInterpreter()
        self.assertTrue(interp.UserCommandExists("foobar"), "foobar exists")
        self.assertFalse(interp.CommandExists("foobar"), "It is not a builtin.")

        # Test a bunch of different kinds of python callables with
        # both 4 and 5 positional arguments.
        self.expect("foobar", substrs=["All good"])
        self.expect("foobar4", substrs=["All good"])
        self.expect("vfoobar", substrs=["All good"])
        self.expect("v5foobar", substrs=["All good"])
        self.expect("sfoobar", substrs=["All good"])
        self.expect("cfoobar", substrs=["All good"])
        self.expect("ifoobar", substrs=["All good"])
        self.expect("sfoobar4", substrs=["All good"])
        self.expect("cfoobar4", substrs=["All good"])
        self.expect("ifoobar4", substrs=["All good"])
        self.expect("ofoobar", substrs=["All good"])
        self.expect("ofoobar4", substrs=["All good"])

        # Verify command that specifies eCommandRequiresTarget returns failure
        # without a target.
        self.expect("targetname", substrs=["a.out"], matching=False, error=True)

        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out"])

        self.expect("targetname", substrs=["a.out"], matching=True, error=False)

        # This is the function to remove the custom commands in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("command script delete welcome", check=False)
            self.runCmd("command script delete targetname", check=False)
            self.runCmd("command script delete longwait", check=False)
            self.runCmd("command script delete mysto", check=False)
            self.runCmd("command script delete tell_sync", check=False)
            self.runCmd("command script delete tell_async", check=False)
            self.runCmd("command script delete tell_curr", check=False)
            self.runCmd("command script delete bug11569", check=False)
            self.runCmd("command script delete takes_exe_ctx", check=False)
            self.runCmd("command script delete decorated", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Interact with debugger in synchronous mode
        self.setAsync(False)

        # We don't want to display the stdout if not in TraceOn() mode.
        if not self.TraceOn():
            self.HideStdout()

        self.expect("welcome Enrico", substrs=["Hello Enrico, welcome to LLDB"])

        self.expect(
            "help welcome",
            substrs=[
                "Just a docstring for welcome_impl",
                "A command that says hello to LLDB users",
            ],
        )

        decorated_commands = ["decorated" + str(n) for n in range(1, 5)]
        for name in decorated_commands:
            self.expect(name, substrs=["hello from " + name])
            self.expect(
                "help " + name, substrs=["Python command defined by @lldb.command"]
            )

        self.expect(
            "help",
            substrs=["For more information run"] + decorated_commands + ["welcome"],
        )

        self.expect(
            "help -a",
            substrs=["For more information run"] + decorated_commands + ["welcome"],
        )

        self.expect("help -u", matching=False, substrs=["For more information run"])

        self.runCmd("command script delete welcome")

        self.expect(
            "welcome Enrico",
            matching=False,
            error=True,
            substrs=["Hello Enrico, welcome to LLDB"],
        )

        self.expect(
            "targetname fail", error=True, substrs=["a test for error in command"]
        )

        self.expect(
            "command script list", substrs=["targetname", "For more information run"]
        )

        self.expect(
            "help targetname",
            substrs=["Expects", "'raw'", "input", "help", "raw-input"],
        )

        self.expect("longwait", substrs=["Done; if you saw the delays I am doing OK"])

        self.runCmd("break set -f main.cpp -l 48")
        self.runCmd("run")
        self.runCmd("mysto 3")
        self.expect(
            "frame variable array",
            substrs=["[0] = 79630", "[1] = 388785018", "[2] = 0"],
        )
        self.runCmd("mysto 3")
        self.expect(
            "frame variable array",
            substrs=["[0] = 79630", "[4] = 388785018", "[5] = 0"],
        )

        # we cannot use the stepover command to check for async execution mode since LLDB
        # seems to get confused when events start to queue up
        self.expect("tell_sync", substrs=["running sync"])
        self.expect("tell_async", substrs=["running async"])
        self.expect("tell_curr", substrs=["I am running sync"])

        # check that the execution context is passed in to commands that ask for it
        self.expect("takes_exe_ctx", substrs=["a.out"])

        # Test that a python command can redefine itself
        self.expect('command script add -f foobar welcome -h "just some help"')

        self.runCmd("command script clear")

        # Test that re-defining an existing command works
        self.runCmd("command script add my_command --class welcome.WelcomeCommand")
        self.expect("my_command Blah", substrs=["Hello Blah, welcome to LLDB"])

        self.runCmd(
            "command script add my_command -o --class welcome.TargetnameCommand"
        )
        self.expect("my_command", substrs=["a.out"])

        # Test that without --overwrite we are not allowed to redefine the command.
        self.expect(
            "command script add my_command --class welcome.TargetnameCommand",
            substrs=[
                (
                    'user command "my_command" already exists and force replace was'
                    " not set by --overwrite or 'settings set"
                    " interpreter.require-overwrite false'"
                ),
            ],
            error=True,
        )

        self.runCmd("command script clear")

        self.expect(
            "command script list", matching=False, substrs=["targetname", "longwait"]
        )

        self.expect(
            "command script add -f foobar frame",
            error=True,
            substrs=["cannot add command"],
        )

        # http://llvm.org/bugs/show_bug.cgi?id=11569
        # LLDBSwigPythonCallCommand crashes when a command script returns an
        # object
        self.runCmd("command script add -f bug11569 bug11569")
        # This should not crash.
        self.runCmd("bug11569", check=False)

        # Make sure that a reference to a non-existent class raises an error:
        bad_class_name = "LLDBNoSuchModule.LLDBNoSuchClass"
        self.expect(
            "command script add wont-work --class {0}".format(bad_class_name),
            error=True,
            substrs=[bad_class_name],
        )

    def test_persistence(self):
        """
        Ensure that function arguments meaningfully persist (and do not crash!)
        even after the function terminates.
        """
        self.runCmd("command script import persistence.py")
        self.runCmd("command script add -f persistence.save_debugger save_debugger")
        self.expect("save_debugger", substrs=[str(self.dbg)])

        # After the command completes, the debugger object should still be
        # valid.
        self.expect("script str(persistence.debugger_copy)", substrs=[str(self.dbg)])
        # The result object will be replaced by an empty result object (in the
        # "Started" state).
        self.expect("script str(persistence.result_copy)", substrs=["Started"])

    def test_interactive(self):
        """
        Test that we can add multiple lines interactively.
        """
        interp = self.dbg.GetCommandInterpreter()
        cmd_file = self.getSourcePath("cmd_file.lldb")
        result = lldb.SBCommandReturnObject()
        interp.HandleCommand(f"command source {cmd_file}", result)
        self.assertCommandReturn(result, "Sourcing the command should cause no errors.")
        self.assertTrue(interp.UserCommandExists("my_cmd"), "Command defined.")
        interp.HandleCommand("my_cmd", result)
        self.assertCommandReturn(result, "Running the command succeeds")
        self.assertIn("My Command Result", result.GetOutput(), "Command was correct")



"""
Test Lua API wrapper
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import subprocess


def to_string(b):
    """Return the parameter as type 'str', possibly encoding it.

    In Python2, the 'str' type is the same as 'bytes'. In Python3, the
    'str' type is (essentially) Python2's 'unicode' type, and 'bytes' is
    distinct.

    """
    if isinstance(b, str):
        # In Python2, this branch is taken for types 'str' and 'bytes'.
        # In Python3, this branch is taken only for 'str'.
        return b
    if isinstance(b, bytes):
        # In Python2, this branch is never taken ('bytes' is handled as 'str').
        # In Python3, this is true only for 'bytes'.
        try:
            return b.decode("utf-8")
        except UnicodeDecodeError:
            # If the value is not valid Unicode, return the default
            # repr-line encoding.
            return str(b)

    # By this point, here's what we *don't* have:
    #
    #  - In Python2:
    #    - 'str' or 'bytes' (1st branch above)
    #  - In Python3:
    #    - 'str' (1st branch above)
    #    - 'bytes' (2nd branch above)
    #
    # The last type we might expect is the Python2 'unicode' type. There is no
    # 'unicode' type in Python3 (all the Python3 cases were already handled). In
    # order to get a 'str' object, we need to encode the 'unicode' object.
    try:
        return b.encode("utf-8")
    except AttributeError:
        raise TypeError("not sure how to convert %s to %s" % (type(b), str))


class ExecuteCommandTimeoutException(Exception):
    def __init__(self, msg, out, err, exitCode):
        assert isinstance(msg, str)
        assert isinstance(out, str)
        assert isinstance(err, str)
        assert isinstance(exitCode, int)
        self.msg = msg
        self.out = out
        self.err = err
        self.exitCode = exitCode


# Close extra file handles on UNIX (on Windows this cannot be done while
# also redirecting input).
kUseCloseFDs = not (platform.system() == "Windows")


def executeCommand(command, cwd=None, env=None, input=None, timeout=0):
    """Execute command ``command`` (list of arguments or string) with.

    * working directory ``cwd`` (str), use None to use the current
    working directory
    * environment ``env`` (dict), use None for none
    * Input to the command ``input`` (str), use string to pass
    no input.
    * Max execution time ``timeout`` (int) seconds. Use 0 for no timeout.

    Returns a tuple (out, err, exitCode) where
    * ``out`` (str) is the standard output of running the command
    * ``err`` (str) is the standard error of running the command
    * ``exitCode`` (int) is the exitCode of running the command

    If the timeout is hit an ``ExecuteCommandTimeoutException``
    is raised.

    """
    if input is not None:
        input = to_bytes(input)
    p = subprocess.Popen(
        command,
        cwd=cwd,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        env=env,
        close_fds=kUseCloseFDs,
    )
    timerObject = None
    # FIXME: Because of the way nested function scopes work in Python 2.x we
    # need to use a reference to a mutable object rather than a plain
    # bool. In Python 3 we could use the "nonlocal" keyword but we need
    # to support Python 2 as well.
    hitTimeOut = [False]
    try:
        if timeout > 0:

            def killProcess():
                # We may be invoking a shell so we need to kill the
                # process and all its children.
                hitTimeOut[0] = True
                killProcessAndChildren(p.pid)

            timerObject = threading.Timer(timeout, killProcess)
            timerObject.start()

        out, err = p.communicate(input=input)
        exitCode = p.wait()
    finally:
        if timerObject is not None:
            timerObject.cancel()

    # Ensure the resulting output is always of string type.
    out = to_string(out)
    err = to_string(err)

    if hitTimeOut[0]:
        raise ExecuteCommandTimeoutException(
            msg="Reached timeout of {} seconds".format(timeout),
            out=out,
            err=err,
            exitCode=exitCode,
        )

    # Detect Ctrl-C in subprocess.
    if exitCode == -signal.SIGINT:
        raise KeyboardInterrupt

    return out, err, exitCode


class TestLuaAPI(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def get_tests(self):
        tests = []
        for filename in os.listdir():
            # Ignore dot files and excluded tests.
            if filename.startswith("."):
                continue

            # Ignore files that don't start with 'Test'.
            if not filename.startswith("Test"):
                continue

            if not os.path.isdir(filename):
                base, ext = os.path.splitext(filename)
                if ext == ".lua":
                    tests.append(filename)
        return tests

    @skipIfMTE  # Lua is not MTE-aware.
    def test_lua_api(self):
        if "LUA_EXECUTABLE" not in os.environ or not os.path.exists(
            os.environ["LUA_EXECUTABLE"]
        ):
            self.skipTest("Lua API tests could not find Lua executable.")
            return
        lua_executable = os.environ["LUA_EXECUTABLE"]
        lldb_lua_cpath = os.environ["LLDB_LUA_CPATH"]

        self.build()
        test_exe = self.getBuildArtifact("a.out")
        test_output = self.getBuildArtifact("output")
        test_input = self.getBuildArtifact("input")

        lua_prelude = "package.cpath = '%s/?.so;' .. package.cpath" % lldb_lua_cpath

        lua_env = {
            "TEST_EXE": os.path.join(self.getBuildDir(), test_exe),
            "TEST_OUTPUT": os.path.join(self.getBuildDir(), test_output),
            "TEST_INPUT": os.path.join(self.getBuildDir(), test_input),
        }

        for lua_test in self.get_tests():
            cmd = [lua_executable] + ["-e", lua_prelude] + [lua_test]
            out, err, exitCode = executeCommand(cmd, env=lua_env)

            # Redirect Lua output
            print(out)
            print(err, file=sys.stderr)

            self.assertEqual(exitCode, 0, "Lua test '%s' failure." % lua_test)



"""Check that compiler-generated constant values work correctly"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ConstVariableTestCase(TestBase):
    @expectedFailureAll(oslist=["freebsd", "linux"], compiler="icc")
    @expectedFailureAll(archs=["mips", "mipsel", "mips64", "mips64el"])
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24489: Name lookup not working correctly on Windows",
    )
    def test_and_run_command(self):
        """Test interpreted and JITted expressions on constant values."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break inside the main.
        lldbutil.run_break_set_by_symbol(self, "main", num_expected_locations=1)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        self.runCmd("next")
        self.runCmd("next")

        # Try frame variable.
        self.expect(
            "frame variable index",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["(int32_t) index = 512"],
        )

        # Try an interpreted expression.
        self.expect(
            "expr (index + 512)", VARIABLES_DISPLAYED_CORRECTLY, substrs=["1024"]
        )

        # Try a JITted expression.
        self.expect(
            "expr (int)getpid(); (index - 256)",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["256"],
        )

        self.runCmd("kill")



"""Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test_event.build_exception import BuildError


class StepUntilTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "main.c"
        self.less_than_two = line_number("main.c", "Less than 2")
        self.greater_than_two = line_number("main.c", "Greater than or equal to 2.")
        self.back_out_in_main = line_number("main.c", "Back out in main")
        self.in_foo = line_number("main.c", "In foo")

    def _build_dict_for_discontinuity(self):
        return dict(
            CFLAGS_EXTRAS="-funique-basic-block-section-names "
            + "-ffunction-sections -fbasic-block-sections=list="
            + self.getSourcePath("function.list"),
            LD_EXTRAS="-Wl,--script=" + self.getSourcePath("symbol.order"),
        )

    def _common_setup(self, build_dict, args):
        self.build(dictionary=build_dict)
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        main_source_spec = lldb.SBFileSpec(self.main_source)
        break_before = target.BreakpointCreateBySourceRegex(
            "At the start", main_source_spec
        )
        self.assertTrue(break_before, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(args, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_before)

        if len(threads) != 1:
            self.fail("Failed to stop at first breakpoint in main.")

        thread = threads[0]
        return thread

    def do_until(self, args, until_lines, expected_linenum):
        thread = self._common_setup(None, args)

        cmd_interp = self.dbg.GetCommandInterpreter()
        ret_obj = lldb.SBCommandReturnObject()

        cmd_line = "thread until"
        for line_num in until_lines:
            cmd_line += " %d" % (line_num)

        cmd_interp.HandleCommand(cmd_line, ret_obj)
        self.assertTrue(
            ret_obj.Succeeded(), "'%s' failed: %s." % (cmd_line, ret_obj.GetError())
        )

        frame = thread.frames[0]
        line = frame.GetLineEntry().GetLine()
        self.assertEqual(
            line, expected_linenum, "Did not get the expected stop line number"
        )

    def test_hitting_one(self):
        """Test thread step until - targeting one line and hitting it."""
        self.do_until(None, [self.less_than_two], self.less_than_two)

    def test_targetting_two_hitting_first(self):
        """Test thread step until - targeting two lines and hitting one."""
        self.do_until(
            ["foo", "bar", "baz"],
            [self.less_than_two, self.greater_than_two],
            self.greater_than_two,
        )

    def test_targetting_two_hitting_second(self):
        """Test thread step until - targeting two lines and hitting the other one."""
        self.do_until(
            None, [self.less_than_two, self.greater_than_two], self.less_than_two
        )

    def test_missing_one(self):
        """Test thread step until - targeting one line and missing it by stepping out to call site"""
        self.do_until(
            ["foo", "bar", "baz"], [self.less_than_two], self.back_out_in_main
        )

    @no_debug_info_test
    def test_bad_line(self):
        """Test that we get an error if attempting to step outside the current
        function"""
        thread = self._common_setup(None, None)
        self.expect(
            f"thread until {self.in_foo}",
            substrs=["Until target outside of the current function"],
            error=True,
        )

    @no_debug_info_test
    @skipIf(oslist=lldbplatformutil.getDarwinOSTriples() + ["windows"])
    @skipIf(archs=no_match(["x86_64", "aarch64"]))
    @skipIf(compiler=no_match(["clang"]))
    def test_bad_line_discontinuous(self):
        """Test that we get an error if attempting to step outside the current
        function -- and the function is discontinuous"""
        try:
            self.build(dictionary=self._build_dict_for_discontinuity())
        except BuildError as ex:
            self.skipTest(f"failed to build with linker script.")

        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "At the start", lldb.SBFileSpec(self.main_source)
        )
        self.expect(
            f"thread until {self.in_foo}",
            substrs=["Until target outside of the current function"],
            error=True,
        )



# RUN: %PYTHON %s | FileCheck %s

# Naming this file with a `_dialect` suffix to avoid a naming conflict with
# python package's math module (coming in from random.py).

from mlir.ir import *
import mlir.dialects.func as func
import mlir.dialects.complex as mlir_complex


def run(f):
    print("\nTEST:", f.__name__)
    f()


# CHECK-LABEL: TEST: testComplexOps
@run
def testComplexOps():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(ComplexType.get(F32Type.get()))
            def emit_add(arg):
                return mlir_complex.AddOp(arg, arg)

        # CHECK-LABEL: func @emit_add(
        # CHECK-SAME:                  %[[ARG:.*]]: complex<f32>) -> complex<f32> {
        # CHECK:         %[[RES:.*]] = complex.add %[[ARG]], %[[ARG]] : complex<f32>
        # CHECK:         return %[[RES]] : complex<f32>
        # CHECK:       }
        print(module)



"""
Test lldb command aliases.
"""

import unittest
import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LaunchInTerminalTestCase(TestBase):
    # Darwin is the only platform that I know of that supports optionally launching
    # a program in a separate terminal window. It would be great if other platforms
    # added support for this.
    @skipUnlessDarwin
    # If the test is being run under sudo, the spawned terminal won't retain that elevated
    # privilege so it can't open the socket to talk back to the test case
    @unittest.skipIf(
        hasattr(os, "geteuid") and os.geteuid() == 0, "test cannot be run as root"
    )
    # Do we need to disable this test if the testsuite is being run on a remote system?
    # This env var is only defined when the shell is running in a local mac
    # terminal window
    @unittest.skipUnless(
        "TERM_PROGRAM" in os.environ, "test must be run on local system"
    )
    @no_debug_info_test
    def test_launch_in_terminal(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        launch_info = lldb.SBLaunchInfo(["-lAF", "/tmp/"])
        launch_info.SetLaunchFlags(
            lldb.eLaunchFlagLaunchInTTY | lldb.eLaunchFlagCloseTTYOnExit
        )
        error = lldb.SBError()
        process = target.Launch(launch_info, error)
        print("Error was: %s." % (error.GetCString()))
        self.assertTrue(
            error.Success(),
            "Make sure launch happened successfully in a terminal window",
        )
        # Running in synchronous mode our process should have run and already
        # exited by the time target.Launch() returns
        self.assertState(process.GetState(), lldb.eStateExited)



"""Test that lldb functions correctly after the inferior has crashed."""


import lldb
from lldbsuite.test import lldbutil
from lldbsuite.test import lldbplatformutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class CrashingInferiorTestCase(TestBase):
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    @expectedFailureNetBSD
    def test_inferior_crashing(self):
        """Test that lldb reliably catches the inferior crashing (command)."""
        self.build()
        self.inferior_crashing()

    def test_inferior_crashing_register(self):
        """Test that lldb reliably reads registers from the inferior after crashing (command)."""
        self.build()
        self.inferior_crashing_registers()

    @add_test_categories(["pyapi"])
    def test_inferior_crashing_python(self):
        """Test that lldb reliably catches the inferior crashing (Python API)."""
        self.build()
        self.inferior_crashing_python()

    def test_inferior_crashing_expr(self):
        """Test that the lldb expression interpreter can read from the inferior after crashing (command)."""
        self.build()
        self.inferior_crashing_expr()

    def set_breakpoint(self, line):
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", line, num_expected_locations=1, loc_exact=True
        )

    def check_stop_reason(self):
        # We should have one crashing thread
        self.assertEqual(
            len(
                lldbutil.get_crashed_threads(
                    self, self.dbg.GetSelectedTarget().GetProcess()
                )
            ),
            1,
            STOPPED_DUE_TO_EXC_BAD_ACCESS,
        )

    def get_api_stop_reason(self):
        return lldb.eStopReasonException

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number of the crash.
        self.line = line_number("main.c", "// Crash here.")

    def inferior_crashing(self):
        """Inferior crashes upon launching; lldb should catch the event and stop."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        self.runCmd("run", RUN_SUCCEEDED)
        # The exact stop reason depends on the platform
        if self.platformIsDarwin():
            stop_reason = "stop reason = EXC_BAD_ACCESS"
        elif self.getPlatform() == "linux" or self.getPlatform() == "freebsd":
            stop_reason = "stop reason = signal SIGSEGV: address not mapped to object"
        else:
            stop_reason = "stop reason = invalid address"
        self.expect(
            "thread list",
            STOPPED_DUE_TO_EXC_BAD_ACCESS,
            substrs=["stopped", stop_reason],
        )

        # And it should report the correct line number.
        self.expect(
            "thread backtrace all", substrs=[stop_reason, "main.c:%d" % self.line]
        )

    def inferior_crashing_python(self):
        """Inferior crashes upon launching; lldb should catch the event and stop."""
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now launch the process, and do not stop at entry point.
        # Both argv and envp are null.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if process.GetState() != lldb.eStateStopped:
            self.fail(
                "Process should be in the 'stopped' state, "
                "instead the actual state is: '%s'"
                % lldbutil.state_type_to_str(process.GetState())
            )

        threads = lldbutil.get_crashed_threads(self, process)
        self.assertEqual(
            len(threads), 1, "Failed to stop the thread upon bad access exception"
        )

        if self.TraceOn():
            lldbutil.print_stacktrace(threads[0])

    def inferior_crashing_registers(self):
        """Test that lldb can read registers after crashing."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        self.runCmd("run", RUN_SUCCEEDED)
        self.check_stop_reason()

        # lldb should be able to read from registers from the inferior after
        # crashing.
        lldbplatformutil.check_first_register_readable(self)

    def inferior_crashing_expr(self):
        """Test that the lldb expression interpreter can read symbols after crashing."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        self.runCmd("run", RUN_SUCCEEDED)
        self.check_stop_reason()

        # The lldb expression interpreter should be able to read from addresses
        # of the inferior after a crash.
        self.expect("expression argc", startstr="(int) $0 = 1")

        self.expect("expression hello_world", substrs=["Hello"])



"""
Test the AArch64 SVE and Streaming SVE (SSVE) registers dynamic resize with
multiple threads.

This test assumes a minimum supported vector length (VL) of 256 bits
and will test 512 bits if possible. We refer to "vg" which is the
register shown in lldb. This is in units of 64 bits. 256 bit VL is
the same as a vg of 4.
"""

from enum import Enum
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class Mode(Enum):
    SVE = 0
    SSVE = 1


class RegisterCommandsTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def get_supported_vg(self):
        # Changing VL trashes the register state, so we need to run the program
        # just to test this. Then run it again for the test.
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        main_thread_stop_line = line_number("main.c", "// Break in main thread")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", main_thread_stop_line)

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread info 1",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
        )

        # Write back the current vg to confirm read/write works at all.
        current_vg = self.match("register read vg", ["(0x[0-9]+)"])
        self.assertIsNotNone(current_vg)
        self.expect("register write vg {}".format(current_vg.group()))

        # Aka 128, 256 and 512 bit.
        supported_vg = []
        for vg in [2, 4, 8]:
            # This could mask other errors but writing vg is tested elsewhere
            # so we assume the hardware rejected the value.
            self.runCmd("register write vg {}".format(vg), check=False)
            if not self.res.GetError():
                supported_vg.append(vg)

        self.runCmd("breakpoint delete 1")
        self.runCmd("continue")

        return supported_vg

    def check_sve_registers(self, vg_test_value):
        z_reg_size = vg_test_value * 8
        p_reg_size = int(z_reg_size / 8)

        p_value_bytes = ["0xff", "0x55", "0x11", "0x01", "0x00"]

        for i in range(32):
            s_reg_value = "s%i = 0x" % (i) + "".join(
                "{:02x}".format(i + 1) for _ in range(4)
            )

            d_reg_value = "d%i = 0x" % (i) + "".join(
                "{:02x}".format(i + 1) for _ in range(8)
            )

            v_reg_value = "v%i = 0x" % (i) + "".join(
                "{:02x}".format(i + 1) for _ in range(16)
            )

            z_reg_value = (
                "{"
                + " ".join("0x{:02x}".format(i + 1) for _ in range(z_reg_size))
                + "}"
            )

            self.expect("register read -f hex " + "s%i" % (i), substrs=[s_reg_value])

            self.expect("register read -f hex " + "d%i" % (i), substrs=[d_reg_value])

            self.expect("register read -f hex " + "v%i" % (i), substrs=[v_reg_value])

            self.expect("register read " + "z%i" % (i), substrs=[z_reg_value])

        for i in range(16):
            p_regs_value = (
                "{" + " ".join(p_value_bytes[i % 5] for _ in range(p_reg_size)) + "}"
            )
            self.expect("register read " + "p%i" % (i), substrs=[p_regs_value])

        self.expect("register read ffr", substrs=[p_regs_value])

    def build_for_mode(self, mode):
        cflags = "-march=armv8-a+sve -lpthread"
        if mode == Mode.SSVE:
            cflags += " -DUSE_SSVE"
        self.build(dictionary={"CFLAGS_EXTRAS": cflags})

    def run_sve_test(self, mode):
        if (mode == Mode.SVE) and not self.isAArch64SVE():
            self.skipTest("SVE registers must be supported.")

        if (mode == Mode.SSVE) and not self.isAArch64SMEFA64():
            self.skipTest(
                "Streaming SVE registers must be supported and the "
                "smefa64 extension must be present."
            )

        self.build_for_mode(mode)

        supported_vg = self.get_supported_vg()

        if not (2 in supported_vg and 4 in supported_vg):
            self.skipTest("Not all required SVE vector lengths are supported.")

        main_thread_stop_line = line_number("main.c", "// Break in main thread")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", main_thread_stop_line)

        thX_break_line1 = line_number("main.c", "// Thread X breakpoint 1")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", thX_break_line1)

        thX_break_line2 = line_number("main.c", "// Thread X breakpoint 2")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", thX_break_line2)

        thY_break_line1 = line_number("main.c", "// Thread Y breakpoint 1")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", thY_break_line1)

        thY_break_line2 = line_number("main.c", "// Thread Y breakpoint 2")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", thY_break_line2)

        self.runCmd("run", RUN_SUCCEEDED)

        process = self.dbg.GetSelectedTarget().GetProcess()

        self.expect(
            "thread info 1",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
        )

        if 8 in supported_vg:
            self.check_sve_registers(8)
        else:
            self.check_sve_registers(4)

        self.runCmd("process continue", RUN_SUCCEEDED)

        for idx in range(1, process.GetNumThreads()):
            thread = process.GetThreadAtIndex(idx)
            if thread.GetStopReason() != lldb.eStopReasonBreakpoint:
                self.runCmd("thread continue %d" % (idx + 1))
                self.assertEqual(thread.GetStopReason(), lldb.eStopReasonBreakpoint)

            stopped_at_line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()

            if stopped_at_line_number == thX_break_line1:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(4)
                self.runCmd("register write vg 2")

            elif stopped_at_line_number == thY_break_line1:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(2)
                self.runCmd("register write vg 4")

        self.runCmd("thread continue 2")
        self.runCmd("thread continue 3")

        for idx in range(1, process.GetNumThreads()):
            thread = process.GetThreadAtIndex(idx)
            self.assertEqual(thread.GetStopReason(), lldb.eStopReasonBreakpoint)

            stopped_at_line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()

            if stopped_at_line_number == thX_break_line2:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(2)

            elif stopped_at_line_number == thY_break_line2:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(4)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_sve_registers_dynamic_config(self):
        """Test AArch64 SVE registers multi-threaded dynamic resize."""
        self.run_sve_test(Mode.SVE)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_ssve_registers_dynamic_config(self):
        """Test AArch64 SSVE registers multi-threaded dynamic resize."""
        self.run_sve_test(Mode.SSVE)

    def setup_svg_test(self, mode):
        # Even when running in SVE mode, we need access to SVG for these tests.
        if not self.isAArch64SMEFA64():
            self.skipTest(
                "Streaming SVE registers must be present and the "
                "smefa64 extension must be present."
            )

        self.build_for_mode(mode)

        supported_vg = self.get_supported_vg()

        main_thread_stop_line = line_number("main.c", "// Break in main thread")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", main_thread_stop_line)

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread info 1",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
        )

        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        return process, supported_vg

    def read_reg(self, process, regset, reg):
        registerSets = process.GetThreadAtIndex(0).GetFrameAtIndex(0).GetRegisters()
        sve_registers = registerSets.GetFirstValueByName(regset)
        return sve_registers.GetChildMemberWithName(reg).GetValueAsUnsigned()

    def read_vg(self, process):
        return self.read_reg(process, "Scalable Vector Extension Registers", "vg")

    def read_svg(self, process):
        return self.read_reg(process, "Scalable Matrix Extension Registers", "svg")

    def do_svg_test(self, process, vgs, expected_svgs):
        for vg, svg in zip(vgs, expected_svgs):
            self.runCmd("register write vg {}".format(vg))
            self.assertEqual(svg, self.read_svg(process))

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_svg_sve_mode(self):
        """When in SVE mode, svg should remain constant as we change vg."""
        process, supported_vg = self.setup_svg_test(Mode.SVE)
        svg = self.read_svg(process)
        self.do_svg_test(process, supported_vg, [svg] * len(supported_vg))

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_svg_ssve_mode(self):
        """When in SSVE mode, changing vg should change svg to the same value."""
        process, supported_vg = self.setup_svg_test(Mode.SSVE)
        self.do_svg_test(process, supported_vg, supported_vg)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_sme_not_present(self):
        """When there is no SME, we should not show the SME register sets."""
        if self.isAArch64SME():
            self.skipTest("Streaming SVE registers must not be present.")

        self.build_for_mode(Mode.SVE)

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This test may run on a non-sve system, but we'll stop before any
        # SVE instruction would be run.
        self.runCmd("b main")
        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread info 1",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
        )

        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        registerSets = process.GetThreadAtIndex(0).GetFrameAtIndex(0).GetRegisters()
        sme_registers = registerSets.GetFirstValueByName(
            "Scalable Matrix Extension Registers"
        )
        self.assertFalse(sme_registers.IsValid())



"""
Test the BoundsSafety instrumentation plugin
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


STOP_REASON_MAX_LEN = 100
SOFT_TRAP_FUNC_MINIMAL = "__bounds_safety_soft_trap"
SOFT_TRAP_FUNC_WITH_STR = "__bounds_safety_soft_trap_s"


class BoundsSafetyTestSoftTrapPlugin(TestBase):
    SHARED_BUILD_TESTCASE = False

    def _check_stop_reason_impl(
        self,
        expected_soft_trap_func: str,
        expected_stop_reason: str,
        expected_func_name: str,
        expected_file_name: str,
        expected_line_num: int,
    ):
        process = self.test_target.process
        thread = process.GetSelectedThread()
        self.assertEqual(
            thread.GetStopReason(),
            lldb.eStopReasonInstrumentation,
        )

        stop_reason = thread.GetStopDescription(STOP_REASON_MAX_LEN)
        self.assertEqual(stop_reason, expected_stop_reason)

        soft_trap_func_frame = thread.GetFrameAtIndex(0)
        self.assertEqual(soft_trap_func_frame.name, expected_soft_trap_func)

        stop_frame = thread.GetSelectedFrame()
        self.assertEqual(stop_frame.name, expected_func_name)
        # The stop frame isn't frame 1 because that frame is the artificial
        # frame containing the trap reason.
        self.assertEqual(stop_frame.idx, 2)
        file_name = stop_frame.GetLineEntry().GetFileSpec().basename
        self.assertEqual(file_name, expected_file_name)
        line = stop_frame.GetLineEntry().line
        self.assertEqual(line, expected_line_num)

    def check_state_soft_trap_minimal(
        self, stop_reason: str, func_name: str, file_name: str, line_num: int
    ):
        """
        Check the program state is as expected when hitting
        a soft trap from -fbounds-safety-soft-traps=call-minimal
        """
        self._check_stop_reason_impl(
            SOFT_TRAP_FUNC_MINIMAL,
            expected_stop_reason=stop_reason,
            expected_func_name=func_name,
            expected_file_name=file_name,
            expected_line_num=line_num,
        )

    def check_state_soft_trap_with_str(
        self, stop_reason: str, func_name: str, file_name: str, line_num: int
    ):
        """
        Check the program state is as expected when hitting
        a soft trap from -fbounds-safety-soft-traps=call-with_str
        """
        self._check_stop_reason_impl(
            SOFT_TRAP_FUNC_WITH_STR,
            expected_stop_reason=stop_reason,
            expected_func_name=func_name,
            expected_file_name=file_name,
            expected_line_num=line_num,
        )

    # Skip the tests on Windows because they fail due to the stop reason
    # being `eStopReasonNon` instead of the expected
    # `eStopReasonInstrumentation`.
    @skipIfWindows
    @skipUnlessBoundsSafety
    def test_call_minimal(self):
        """
        Test the plugin on code built with
        -fbounds-safety-soft-traps=call-minimal
        """
        self.build(make_targets=["soft-trap-test-minimal"])
        self.test_target = self.createTestTarget()
        self.runCmd("run")

        process = self.test_target.process

        # First soft trap hit
        self.check_state_soft_trap_minimal(
            "Soft Bounds check failed: indexing above upper bound in 'buffer[2]'",
            "main",
            "main.c",
            7,
        )

        process.Continue()

        # Second soft trap hit
        self.check_state_soft_trap_minimal(
            "Soft Bounds check failed: indexing below lower bound in 'buffer[-1]'",
            "main",
            "main.c",
            8,
        )

        process.Continue()
        self.assertEqual(process.GetState(), lldb.eStateExited)
        self.assertEqual(process.GetExitStatus(), 0)

    @skipIfWindows
    @skipUnlessBoundsSafety
    def test_call_with_str(self):
        """
        Test the plugin on code built with
        -fbounds-safety-soft-traps=call-with-str
        """
        self.build(make_targets=["soft-trap-test-with-str"])
        self.test_target = self.createTestTarget()
        self.runCmd("run")

        process = self.test_target.process

        # First soft trap hit
        self.check_state_soft_trap_with_str(
            "Soft Bounds check failed: indexing above upper bound in 'buffer[2]'",
            "main",
            "main.c",
            7,
        )

        process.Continue()

        # Second soft trap hit
        self.check_state_soft_trap_with_str(
            "Soft Bounds check failed: indexing below lower bound in 'buffer[-1]'",
            "main",
            "main.c",
            8,
        )

        process.Continue()
        self.assertEqual(process.GetState(), lldb.eStateExited)
        self.assertEqual(process.GetExitStatus(), 0)



"""
Test debug-info parsing of synthesized Objective-C properties.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestSynthesizedPropertyAccessor(TestBase):
    def test(self):
        self.build()

        (target, _, _, _) = lldbutil.run_to_source_breakpoint(
            self, "return f.fooProp", lldb.SBFileSpec("main.m")
        )

        getters = target.FindFunctions("-[Foo fooProp]", lldb.eFunctionNameTypeSelector)
        self.assertEqual(len(getters), 1)
        getter = getters[0].function.GetType()
        self.assertTrue(getter)
        self.assertEqual(getter.GetDisplayTypeName(), "int ()")

        setters = target.FindFunctions(
            "-[Foo setFooProp:]", lldb.eFunctionNameTypeSelector
        )
        self.assertEqual(len(setters), 1)
        setter = setters[0].function.GetType()
        self.assertTrue(setter)
        self.assertEqual(setter.GetDisplayTypeName(), "void (int)")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.m"))

        log = self.getBuildArtifact("expr.log")
        self.runCmd(f"log enable lldb expr -f {log}")

        self.expect(
            "vo pair",
            substrs=[
                "warning: `po` was unsuccessful, running `p` instead\n",
                "(Pair) pair = (f = 2, e = 3)",
            ],
        )
        self.filecheck_log(log, __file__, f"-check-prefix=CHECK-VO")
        # CHECK-VO: Object description fallback due to error: not a pointer type

        self.expect(
            "expr -O -- pair",
            substrs=[
                "warning: `po` was unsuccessful, running `p` instead\n",
                "(Pair)  (f = 2, e = 3)",
            ],
        )
        self.filecheck_log(log, __file__, f"-check-prefix=CHECK-EXPR")
        # CHECK-EXPR: Object description fallback due to error: not a pointer type
